{

open Parser ;;

let contents buf =
  let s = Buffer.contents buf in 
  Buffer.reset buf ; s ;;

let buf = Buffer.create 16 ;;

let keyword_table = Hashtbl.create 17 ;;

let _ = List.iter (fun (s, t) -> Hashtbl.add keyword_table s t) [
  ("array", ARRAY); ("begin", BEGIN); ("div", DIV); ("do", DO);
  ("done", DONE); ("end", END); ("else", ELSE); ("false", FALSE);
  ("for", FOR); ("fun", FUN); ("if", IF); ("in", IN); ("mod", MOD);
  ("not", NOT); ("step", STEP); ("then", THEN); ("to", TO);
  ("true", TRUE); ("type", TYPE); ("var", VAR); ("while", WHILE)
] ;;

let ident_or_keyword s =
  try Hashtbl.find keyword_table s
  with Not_found -> IDENT s ;;

}

let ident = ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']*
let digit = ['0'-'9']
let int = digit+
let float = digit+ '.' digit* (('e'|'E') ('+'|'-')? digit+)?
let space = [' ' '\n' '\r' '\t']

let escaped_char = '\\' ['\\' '\'' 'n' 'r' 't' ]
let regular_char = [^ '\\' '\'']
let comment_char = '\'' (escaped_char | regular_char) '\''

rule token = parse
| space+  { token lexbuf }
| "(*"    { comment lexbuf ; token lexbuf }
| int     { INT(int_of_string(Lexing.lexeme lexbuf)) }
| float   { FLOAT(float_of_string(Lexing.lexeme lexbuf)) }
| '\''    { char lexbuf }
| '"'     { string lexbuf }
| ident   { ident_or_keyword(Lexing.lexeme lexbuf) }
| '('     { LPAR }
| ')'     { RPAR }
| '['     { LBRK }
| ']'     { RBRK }
| '{'     { LBRC }
| '}'     { RBRC }
| '.'     { DOT }
| ','     { COMA }
| ':'     { COLON }
| ';'     { SEMI }
| ":="    { COLONEQ }
| '+'     { PLUS }
| '-'     { MINUS }
| '*'     { TIMES }
| '/'     { SLASH }
| '='     { EQ }
| "<>"    { NE }
| '<'     { LT }
| "<="    { LE }
| '>'     { GT }
| ">="    { GE }
| "&&"    { AND }
| "||"    { OR }
| eof     { EOF }
| _       { raise Parsing.Parse_error }

and char = parse
| "\\n'"  { CHAR '\n' }
| "\\r'"  { CHAR '\r' }
| "\\t'"  { CHAR '\t' }
| "\\\\'" { CHAR '\'' }
| "\\''"  { CHAR '\\' }
| "\\\'"
| "\'\'"  { raise Parsing.Parse_error }
| _ '\''  { CHAR(Lexing.lexeme_char lexbuf 0) }

and string = parse
| "\""    { let s = Buffer.contents buf
            in Buffer.reset buf ; STRING s }
| "\\n"   { Buffer.add_char buf '\n' ; string lexbuf }
| "\\r"   { Buffer.add_char buf '\r' ; string lexbuf }
| "\\t"   { Buffer.add_char buf '\t' ; string lexbuf }
| "\\\\"  { Buffer.add_char buf '\\' ; string lexbuf }
| "\\\""  { Buffer.add_char buf '"' ; string lexbuf }
| '\\'    { raise Parsing.Parse_error}
| eof     { failwith "unterminated string litteral" }
| _       { Buffer.add_char buf (Lexing.lexeme_char lexbuf 0) ; string lexbuf }

and comment = parse
| "*)"    { () }
| "(*"    { comment lexbuf ; comment lexbuf }
| '"'     { comment_string lexbuf ; comment lexbuf }
| comment_char  { comment lexbuf }
| eof     { failwith "unterminated comment 1" }
| _       { comment lexbuf }

and comment_string = parse
| '"'     { () }
| '\\' _  { comment_string lexbuf }
| eof     { failwith "unterminated comment 3" }
| _       { comment_string lexbuf }
